import numpy

#THETA_MAX = 32
#RO_MAX = 50
#RO_MAX = 25

import lsst.afw.image as afwImage
import lsst.afw.display.ds9 as ds9
import hsc.qa as qa

if __name__ == '__main__':

    file = 'CORR-0022374-034.fits'

    exp = afwImage.ExposureF(file)
    mask = exp.getMaskedImage().getMask()
    mask &= mask.getPlaneBitMask("DETECTED")
    mask_arr = mask.getArray()
    mask_arr = numpy.array(mask_arr, dtype=numpy.int32)
    mask = afwImage.ImageI(mask_arr)

    nxImg, nyImg = exp.getMaskedImage().getImage().getDimensions()


#    nAngle = 16
#    nRho = 32
###    nAngle = 200
###    nRho = 200

    if False: # precise version
        houghLimit = 15000
        rMargin = 20.0
        nAngle = 1000
        nRho = 1000

    if True: # experiment
        houghLimit = 10000
        rMargin = 5
        nAngle = 200
        nRho = 200

#    nAngle = 360 
#    nRho = 4650

    hough = qa.Hough(mask, nAngle, nRho)
#RO_MAX = 25

    #himage, ang, dis =  hough.calcHough()

    himage, ang, dis = hough.calcHough()
#    import pudb;pudb.set_trace()


    a = -1.0 * (numpy.cos(ang)/numpy.sin(ang))
    b = dis / numpy.sin(ang)
####    print "y = %g * x + %g" % (a, b)
    
    himage.writeFits('output.fits')
####    print ang, dis

    vHough = himage.getArray()
    nyHough, nxHough = vHough.shape
    print nxHough, nyHough
    vHough1d = numpy.reshape(vHough, nxHough * nyHough )
    x1dInd = numpy.array( [range(nxHough)] * nyHough, dtype=numpy.int32 )
    y1dInd = numpy.array( [ [y] * nxHough for y in range(nyHough) ], dtype=numpy.int32 )
    x1dInd = numpy.reshape(x1dInd, nxHough * nyHough )
    y1dInd = numpy.reshape(y1dInd, nxHough * nyHough )

    # sort in decending order over the whole pixles of hough_image
    ind1dSorted = numpy.argsort(vHough1d) 
    # revert to make it descending
    ind1dSorted = ind1dSorted[-1::-1]
    vHough1dSorted = vHough1d[ind1dSorted]
    x1dIndSorted = x1dInd[ind1dSorted]
    y1dIndSorted = y1dInd[ind1dSorted]
    
    vHoughSorted = numpy.reshape(vHough1dSorted, (nyHough, nxHough))

    himageSorted = afwImage.ImageI(vHoughSorted)
    ds9.mtv(himage, frame=0)
    ds9.mtv(himageSorted, frame=1)

#    import pudb; pudb.set_trace()    

    nCand = len(vHough1dSorted)
    print 'candiates before filtering:', nCand

    xPrev = -numpy.float('inf')
    yPrev = -numpy.float('inf')
    xCandList = numpy.array([])
    yCandList = numpy.array([])
    for (x, y, v) in zip(x1dIndSorted, y1dIndSorted, vHough1dSorted):
        if v < houghLimit :
            continue
        if numpy.hypot(x - xPrev, y - yPrev) > rMargin:
            xCandList = numpy.append(xCandList, x)
            yCandList = numpy.append(yCandList, y)
        xPrev = x
        yPrev = y

#    import pudb;pudb.set_trace()

    nCand = len(xCandList) 
    print 'candiates after filtering:', nCand


    nxHough, nyHough = himage.getDimensions()
    anglePeakList = numpy.pi * xCandList / nAngle
    rhoPeakList = (yCandList - float(nRho)) * int(numpy.hypot(nxImg-1, nyImg-1) + 1) / float(nRho)

    aList = -1.0 * (numpy.cos(anglePeakList)/numpy.sin(anglePeakList))
    bList = rhoPeakList / numpy.sin(anglePeakList)

    if True: # debug
        for (t, r) in zip(xCandList, yCandList):
            ds9.dot('o', t, r, frame=0, size=rMargin, ctype='cyan')
        
        # display the resultant trails on ds9
        ds9.mtv(exp.getMaskedImage().getImage(), frame=2) 
        for (a, b) in zip(aList, bList):
            xx = numpy.array([0, 2048]) 
            yy = a * xx + b 
            points = numpy.array([(x, y) for (x, y) in zip(xx, yy)])
       #    ds9.mtv(himage, frame=0) 
            ds9.line(points, frame=2, ctype='cyan')

# 151.875 0
 
