import numpy, sys 

#THETA_MAX = 32
#RO_MAX = 50
#RO_MAX = 25

import lsst.afw.image as afwImage
import lsst.afw.detection as afwDetection
import lsst.afw.display.ds9 as ds9
import hsc.qa as qa


def getHoughPeaks(himage, minHough, allowedPeakRatio = 1.5, minPeakArea = 10, rMargin = 60.0):
    """
    select all possible peaks (x, y) in resultant Hough-transformed plane
    """
    vHough = himage.getArray()
    nyHough, nxHough = vHough.shape
    print nxHough, nyHough

    # convert hough to 1d array 
    vHough1d = numpy.reshape(vHough, nxHough * nyHough )
    x1dInd = numpy.array( [range(nxHough)] * nyHough, dtype=numpy.int32 )
    y1dInd = numpy.array( [ [y] * nxHough for y in range(nyHough) ], dtype=numpy.int32 )
    x1dInd = numpy.reshape(x1dInd, nxHough * nyHough )
    y1dInd = numpy.reshape(y1dInd, nxHough * nyHough )

    # pick out only high hough pixels
    nCand = len(vHough1d)
    print 'candiates before filtering:', nCand
    if False:
        indHigh = numpy.where(vHough1d > minHough)
    else: # workaround
        indHigh = numpy.logical_and(vHough1d > minHough, y1dInd != 500)
        vHough1dHigh = vHough1d[indHigh]
        x1dIndHigh = x1dInd[indHigh]
        y1dIndHigh = y1dInd[indHigh]

    # grouping high hough-pixel knots, 
    #   assigning one pixel for each knot group by taking the highest hough value in a knot

    xCandList = numpy.array([])
    yCandList = numpy.array([])

    if len(vHough1dHigh) == 0:
        print 'No trail candidate found'
        return xCandList, yCandList 

    ind1dHighSorted = numpy.argsort(vHough1dHigh)
    #   revert to make it descending
    ind1dHighSorted = ind1dHighSorted[-1::-1]
    vHough1dHighSorted = vHough1dHigh[ind1dHighSorted]
    x1dIndHighSorted = x1dIndHigh[ind1dHighSorted]
    y1dIndHighSorted = y1dIndHigh[ind1dHighSorted]

    # further limit pixels by the hough's peak ratio
    maxPeak = vHough1dHighSorted[0]
    minPeak = maxPeak / allowedPeakRatio
    if False:
    #if allowedPeakRatio > 1:
        indLim = numpy.where(vHough1dHighSorted > (peakMax/allowedPeakRatio))
        vHough1dFiltered = vHough1dHighSorted[indLim]
        x1dIndFiltered = x1dIndHighSorted[indLim]
        y1dIndFiltered = y1dIndHighSorted[indLim]
    else:
        vHough1dFiltered = vHough1dHighSorted
        x1dIndFiltered = x1dIndHighSorted
        y1dIndFiltered = y1dIndHighSorted

    #   start grouping knots

    vHoughArr = numpy.copy(vHough1dFiltered)
    xArr = numpy.copy(x1dIndFiltered)
    yArr = numpy.copy(y1dIndFiltered)

    while len(vHoughArr) > 0:
        peakThisKnot = vHoughArr[0]        
        x0 = xArr[0]
        y0 = yArr[0]

        # count up hough points which belongs to this knot group, and
        # carry over the other groups to the next loop
        nPixelsThisKnot = 0
        indOthers = numpy.array([], dtype = int)
        for i, (x, y) in enumerate(zip(xArr, yArr)):
            if numpy.hypot(x - x0, y - y0) < rMargin:
                nPixelsThisKnot += 1
            else:
                indOthers = numpy.append(indOthers, i)

        # decide if this group is a significant trail
        if (peakThisKnot > minPeak) and (nPixelsThisKnot >= minPeakArea):
            xCandList = numpy.append(xCandList, x0)
            yCandList = numpy.append(yCandList, y0)

        vHoughArr = vHoughArr[indOthers]
        xArr = xArr[indOthers]
        yArr = yArr[indOthers]
        
    nCandHigh = len(xCandList)
    print 'candiates after filtering:', nCandHigh

    return xCandList, yCandList

def getTrailCandidates(xHoughPeakList, yHoughPeakList, nxImage, nyImage, nRho, nAngle):
    """
    convert input Hough peaks into line parameters (a:slope, b:intersect)
    """
    
#    nxHough, nyHough = himage.getDimensions()
    anglePeakList = numpy.pi * xHoughPeakList / nAngle
    rhoPeakList = (yHoughPeakList - float(nRho)) * int(numpy.hypot(nxImage-1, nyImage-1) + 1) / float(nRho)

    aList = -1.0 * (numpy.cos(anglePeakList)/numpy.sin(anglePeakList))
    bList = rhoPeakList / numpy.sin(anglePeakList)

    return aList, bList

def maskTrailCandidates(mask, aList, bList, detectedBit, trailBit, trailMargin = 50.0, minTrailArea = 1000):
    """ select footprints of possible trails and set new mask bit to them 
    """

    thresh = afwDetection.Threshold(detectedBit, afwDetection.Threshold.BITMASK)
    fpList = afwDetection.FootprintSet(mask, thresh).getFootprints()

    xmin, ymin = mask.getBBox(afwImage.PARENT).getMin()

    fpTrailList = getTrailFootprints(mask, fpList, aList, bList, trailMargin, minTrailArea)
    
    for fp in fpTrailList:
        for s in fp.getSpans():
            x0, x1 = s.getX0() - xmin, s.getX1() - xmin
            y = s.getY() - ymin
#            x0, x1 = s.getX0(), s.getX1()
#            y = s.getY()
            for x in range(x0, x1):
                mask.set(x, y, mask.get(x, y) | trailBit)

    return fpTrailList

def getTrailFootprints(mask, fpList, aList, bList, trailMargin, minTrailArea):
    """ select footprints of trail candidates from the input footprint list
    """

    # check if this footprint overlaps with the trails candidates in each y 

    xmin, ymin = mask.getBBox(afwImage.PARENT).getMin()

    fpTrailList = []
    for fp in fpList:

        # footprint should be larger than the given minArea to be a trail
        if fp.getArea() < minTrailArea:
            continue

        # check if this fp overlaps with trail candidates
        for s in fp.getSpans():
            x0, x1 = s.getX0() -xmin, s.getX1() -xmin
            y = s.getY() - ymin

            # define trail regions with margin for recognizing this footprint as a candidate
            xTrailList = (y - bList) / aList
            minXTrailList, maxXTrailList  = xTrailList - trailMargin, xTrailList + trailMargin

            if ( numpy.logical_and(minXTrailList < x1, maxXTrailList > x0).any() ):
                fpTrailList.append(fp)
                break

    return fpTrailList


# 151.875 0

def main():

    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('inputFile', help = 'input CORR FITS file to mask trails')
    parser.add_argument('outputFile', help = 'output FITS file name')
    parser.add_argument('--min-hough', dest = 'minHough', type = int, default = 7000, help='minimum peak count in Hough phase space to recognize as a peak (default: 7000)')
    parser.add_argument('--min-peak-area', dest = 'minPeakArea', type = int, default = 10, help='minimum connected pixels above the peak count in Hough phase space to recognize as a peak (default: 10)')
    parser.add_argument('--allowed-peak-ratio', dest = 'allowedPeakRatio', type = float, default = 1.5, help='all the Hough peaks should be larger than the first peak devided by this ratio (default: 1.5)')
    parser.add_argument('--min-trail-area', dest = 'minTrailArea', type = int, default = 300, help='minimum area of each trail candiate (default: 300)')
    parser.add_argument('--trail-margin', dest = 'trailMargin', type = float, default = 30.0, help='acceptable spatial separation of actual trails to be masksed from the trail candidates (default: 30.0)')

    args = parser.parse_args()
    inputFile = args.inputFile
    outputFile = args.outputFile

    minHough = args.minHough
    minPeakArea = args.minPeakArea
    allowedPeakRatio = args.minHough
    minTrailArea = args.minTrailArea
    trailMargin = args.trailMargin

#    file = 'CORR-0022374-034.fits'
#    file = 'CORR-0013200-090.fits'
#    file = 'CORR-0013200-091.fits'
#    file = 'CORR-0013200-096.fits'
#    file = 'CORR-0013200-097.fits' # many lines are detected
#    file = 'CORR-0013202-098.fits'
#    file = 'CORR-0013202-099.fits'

#    nAngle = 16
#    nRho = 32
###    nAngle = 200
###    nRho = 200


    # so far successful Hough parameters
    nAngle = 500
    nRho = 500
    rMargin = 60

    if False: # precise version
        minHough = 15000
        rMargin = 20.0
        nAngle = 1000
        nRho = 1000

    if False: # experiment and successful
#        minHough = 10000
        minHough = 7000
#        minHough = 8000
        minPeakArea = 10
        allowedPeakRatio = 1.5 # 2nd and lower peaks should be higher than (1st peak / allowedPeakRatio)

#        minTrailArea = 1000
#        minTrailArea = 500.0
        minTrailArea = 300
#        minTrailArea = 200.0
#        trailMargin = 50.0
        trailMargin = 30.0

#    nAngle = 360 
#    nRho = 4650

    exp = afwImage.ExposureF(inputFile)
    nxImg, nyImg = exp.getMaskedImage().getImage().getDimensions()

    mask = exp.getMaskedImage().getMask()
    goodMask = mask.getPlaneBitMask("DETECTED")
    badMask = mask.getPlaneBitMask(["NO_DATA", "EDGE", "SAT", "SUSPECT", "INTRP", "BAD"])

#    print badMask
#    mask ^= badMask
#    ds9._mtv(mask, None, 'mask', isMask=False)

    print 'goodMaskBit = %d, badMaskBit = %d' % (goodMask, badMask)
    
    #
    # perform Hough transformation
    #
    maskArr = mask.getArray()
    maskArr = numpy.array(maskArr, dtype=numpy.int32)
    maskHough = afwImage.ImageI(maskArr)

    # emperical scaling parameters based on 2k4k CORR knowledge
    maskEff = maskArr[ 
        numpy.logical_and((maskArr & mask.getPlaneBitMask("NO_DATA") == 0), 
                          (maskArr & mask.getPlaneBitMask("EDGE") == 0)) 
        ]
    if False:
        scale = float(len(maskEff)) / (2048.*4176.)
        if scale < 1: # if the effective area is smaller than 2k4k, we will use 1 as the lowest scale
            scale = 1.0
        minHough *= scale # scaled by effective image area
        print 'effective area: %d (pix) / (2048*4176) = %8.4f : scale_used = %8.4f' % (len(maskEff), float(len(maskEff)) / (2048.*4176.), scale)

    hough = qa.Hough(maskHough, nAngle, nRho)
    himage, ang, dis = hough.calcHough(goodMask, badMask)

    #
    # This is the highest peak returned by in Hough class 
    # But we need to detect all possible peaks corresponding to trails
    #
    a = -1.0 * (numpy.cos(ang)/numpy.sin(ang))
    b = dis / numpy.sin(ang)
    ####    print ang, dis
    ####    print "y = %g * x + %g" % (a, b)

    #
    # detect all possible peaks corresponding to trails
    #
    xHoughPeaks, yHoughPeaks = getHoughPeaks(himage, minHough, allowedPeakRatio, minPeakArea, rMargin)
    aList, bList = getTrailCandidates(xHoughPeaks, yHoughPeaks, nxImg, nyImg, nRho, nAngle)

    # display the resultant Hough peaks on ds9
    if True:
        ds9.mtv(himage, frame = 0)
        for (t, r) in zip(xHoughPeaks, yHoughPeaks):
            ds9.dot('o', t, r, frame = 0, size = rMargin, ctype = 'cyan')
    if False:
        himage.writeFits('hough_output.fits')

    # display the resultant trails on ds9
    if True:
        ds9.mtv(exp.getMaskedImage().getImage(), frame = 1) 
        for (a, b) in zip(aList, bList):
            xx = numpy.array([0, nxImg]) 
            yy = a * xx + b 
            points = numpy.array([(x, y) for (x, y) in zip(xx, yy)])
       #    ds9.mtv(himage, frame=0) 
            ds9.line(points, frame = 1, ctype='cyan')

    #
    # then, mask possible trails' footprints with 'TRAIL' bit
    #
    mask.addMaskPlane('TRAIL')
    trailBit = mask.getPlaneBitMask('TRAIL')
    detectedBit = mask.getPlaneBitMask('DETECTED')

    fpList = maskTrailCandidates(mask, aList, bList, detectedBit, trailBit, trailMargin = trailMargin, minTrailArea = minTrailArea)
    print 'candidate footprint:', len(fpList)
    if False:
        for fp in fpList:
            print fp.getBBox(), fp.getArea()

    exp.writeFits(outputFile)

    # display the resultant mask on ds9
    if True:
        #import pudb;pudb.set_trace()
        ds9.setMaskPlaneColor('TRAIL', 'cyan')
        ds9.mtv(mask.convertU(), frame = 2)
        ds9.mtv(mask, frame = 3)        

if __name__ == '__main__':
    main()

