#!/usr/bin/env python

import lsst.afw.detection as afwDetection
import lsst.afw.image as afwImage

def manipulateMask(mask, detectedBit, addBit):
    """ manipulate mask for footprint with given mask bit on """

    thresh = afwDetection.Threshold(detectedBit, afwDetection.Threshold.BITMASK)
    fpList = afwDetection.FootprintSet(mask, thresh).getFootprints()

    xmin, ymin = mask.getBBox(afwImage.PARENT).getMin()
    width = mask.getWidth()
    for fp in fpList:
        for s in fp.getSpans():

            x0, x1 = s.getX0() - xmin, s.getX1() - xmin
            y = s.getY() - ymin
            for x in range(x0, x1):
                mask.set(x, y, mask.get(x, y) | addBit)


def main():
    file = 'CORR-0013200-090.fits'
    exp = afwImage.ExposureF(file)
    mask = exp.getMaskedImage().getMask()
    detectedBit = mask.getPlaneBitMask('DETECTED')
#    addBit = mask.getPlaneBitMask('BAD')
    addBit = mask.getPlaneBitMask('UNMASKEDNAN')

    manipulateMask(mask, detectedBit, addBit)

    exp.writeFits('modifyMask.fits')
#    import pudb;pudb.set_trace()


if __name__ == '__main__':
    main()

