#include <math.h>
#include "hsc/qa/hough.h"


#define D2R (M_PI/180.0)
#define R2D (180.0/M_PI)

using namespace hsc::qa;

Hough::Hough(lsst::afw::image::Image<int> & image, 
	     int nAngle, 
	     int nRho
	     ) :
  _image(image), 
  _nxImage(image.getDimensions().getX() - 1), 
  _nyImage(image.getDimensions().getY() - 1), 
  _nRho(nRho),
  _nAngle(nAngle), 
  _rhoMax((int) (sqrt(_nxImage*_nxImage + _nyImage*_nyImage) + 1)), 
  _nxHough(nAngle), 
  _nyHough(nRho*2),
  //  himage(NULL)
  _npixImageEff(0)
{
  //  std::cout << "(_nxImage, _nyImage): " << _nxImage << " " << _nyImage  << std::endl;
  _imageHough = lsst::afw::image::Image<int>::Ptr (new lsst::afw::image::Image<int>(_nxHough, _nyHough));
  
}

void Hough::addHough(int x, int y) {
  if(0 <= x && x < _nxHough && 0 <= y && y < _nyHough) {
      (*_imageHough)(x, y) ++;
      //      std::cout << "*_imageHough(x, y): " << (*_imageHough)(x, y) << std::endl;
    }
}


// calcHough with no threshold
lsst::afw::image::Image<int>::Ptr Hough::calcHough(double* anglePeak, double* rhoPeak, int goodVal = 32, int badVal = 0) {
  for(int y = 0; y < _nyImage; ++y) {
    for(int x = 0; x < _nxImage; ++x) {
      int v = _image(x, y);
      //      if((v & bitMask) == 0  && v > 0) {
      if((v & badVal) == 0  && (v & goodVal) > 0) {
	//	std::cout << "image(x, y): " << _image(x, y) << std::endl;
	_npixImageEff ++;
	for(int thetaIndex = 0; thetaIndex < _nAngle; ++thetaIndex) {
	  double theta = M_PI * thetaIndex / _nAngle;
	  double rhoHough = x * cos(theta) + y * sin(theta);
	  int rho = (int) (rhoHough * _nRho / _rhoMax +  0.5);
	  //int rho = (int) (rhoHough * _nRho / _rhoMax);
	  
	  //	  std::cout << "M_PI: " << M_PI << "Theta: " << theta << "cos(theta): " << cos(theta) << "RHO: " << rho << std::endl;

//	  if(rho + _nRho == 500) {
//	    std::cout << "(x, y)=(" << x << "," << y << "), (thetaIndex, theta)=(" << thetaIndex << "," << theta << ")"  << std::endl;
//	  }
	  this->addHough(thetaIndex, rho + _nRho);
	}
      }
    }
  }
  
  int vMax = -1;
  int xmaxHough = 0;
  int ymaxHough = 0;
  for(int y = 0; y < _nyHough; ++y) {
    for(int x = 0; x < _nxHough; ++x) {
      int vHough = (*_imageHough)(x, y);

      //      if(vHough > 2000) {
      //	std::cout << M_PI * x / _nAngle << " " << ((double) y - _nRho ) * _rhoMax / _nRho << std::endl;
      //      }
      //std::cout << vHough << std::endl;      

      if(vHough > vMax) {
	vMax = vHough;
	xmaxHough = x;
	ymaxHough = y;
      }
    }
  }

      //  std::cout << "ymaxHough: " << ymaxHough << std::endl;

  *anglePeak = M_PI * xmaxHough / _nAngle;
  *rhoPeak = ((double) ymaxHough - _nRho ) * _rhoMax / _nRho;

  return _imageHough;

}

// calcHough with image count threshold
lsst::afw::image::Image<int>::Ptr Hough::calcHough(double* anglePeak, double* rhoPeak, lsst::afw::image::Image<float> & imagePixData, float minImage = 10.0, float maxImage = 5000.0, int goodVal = 32, int badVal = 0) {
  for(int y = 0; y < _nyImage; ++y) {
    for(int x = 0; x < _nxImage; ++x) {
      int vMask = _image(x, y);
      float vImage = imagePixData(x, y);
      if(minImage < vImage && vImage < maxImage && (vMask & badVal) == 0 && (vMask & goodVal) > 0) {
	//	std::cout << "image(x, y): " << _image(x, y) << std::endl;
	_npixImageEff ++;
	for(int thetaIndex = 0; thetaIndex < _nAngle; ++thetaIndex) {
	  double theta = M_PI * thetaIndex / _nAngle;
	  double rhoHough = x * cos(theta) + y * sin(theta);
	  int rho = (int) (rhoHough * _nRho / _rhoMax +  0.5);

	  //	  std::cout << "M_PI: " << M_PI << "Theta: " << theta << "cos(theta): " << cos(theta) << "RHO: " << rho << std::endl;

//	  if(rho + _nRho == 500) {
//	    std::cout << "(x, y)=(" << x << "," << y << "), (thetaIndex, theta)=(" << thetaIndex << "," << theta << ")"  << std::endl;
//	  }
	  this->addHough(thetaIndex, rho + _nRho);
	}
      }
    }
  }
  
  int vMax = -1;
  int xmaxHough = 0;
  int ymaxHough = 0;
  for(int y = 0; y < _nyHough; ++y) {
    for(int x = 0; x < _nxHough; ++x) {
      int vHough = (*_imageHough)(x, y);

      //      if(vHough > 2000) {
      //	std::cout << M_PI * x / _nAngle << " " << ((double) y - _nRho ) * _rhoMax / _nRho << std::endl;
      //      }
      //std::cout << vHough << std::endl;      

      if(vHough > vMax) {
	vMax = vHough;
	xmaxHough = x;
	ymaxHough = y;
      }
    }
  }

      //  std::cout << "ymaxHough: " << ymaxHough << std::endl;

  *anglePeak = M_PI * xmaxHough / _nAngle;
  *rhoPeak = ((double) ymaxHough - _nRho ) * _rhoMax / _nRho;

  return _imageHough;

}


