#!/usr/bin/env python

import sys
import math, numpy, os
import lsst.daf.persistence as dafPersist
import lsst.obs.hsc as obsHsc
import lsst.afw.image as afwImage
import lsst.afw.geom as afwGeom

import hsc.ghost.hsc_ghost as hscGhost


def main(fileName, fileName49, root, display = False, outputReg = False, frame = 0, maskMustache = True, maskCaustic = True):
    """  masking 'caustic' ghosts 
    """

    # derive properties of this CCD
    butler = dafPersist.ButlerFactory(mapper=obsHsc.HscMapper(root = root)).create()
    # dataRef = butler.dataRef("calexp", dataId = {'visit':1228, 'ccd':97, 'filter':'HSC-I'})

    exp = afwImage.ExposureF(fileName)
    md = exp.getMetadata()
    ccd = int(md.get('DET-ID'))
    visit = int(md.get('VISIT'))
    filter01 = md.get('FILTER01').upper()
    dataRef = butler.dataRef("calexp", dataId = {'visit': visit, 'ccd': ccd, 'filter': filter01})

    # center coords of this shot
    exp49 = afwImage.ExposureF(fileName49)
    md49 = exp49.getMetadata()
    wcs49 = exp49.getWcs()

    # CRPIX offset between InR center and nominal center (measure-nominal)
    #AXOFF_X = 10.5
    #AXOFF_Y = 9.5 
    # CRPIX offset between ghost center and nominal center (measure-nominal)
    #AXOFF_X = -8.3
    #AXOFF_Y = 53.3 
    # CRPIX offset tweek by hand
    #AXOFF_X = -4
    #AXOFF_Y = 85
    ##    AXOFF_X = -10
    ##    AXOFF_Y = 78 
#    AXOFF_X = -25

    # mustache
    if maskMustache:
        AXOFF_X = 0.
        AXOFF_Y = 0.

        xFovCen_ccd49 = 1036. + AXOFF_X # fov center in ccd49 pixel coords
        yFovCen_ccd49 = 4071. + AXOFF_Y
        fovCenterCoord = wcs49.pixelToSky(afwGeom.PointD(xFovCen_ccd49, yFovCen_ccd49))

        mustache = hscGhost.HscMustacheGhostMaskTask()
        mustache.run(dataRef, exp, centerCoord = fovCenterCoord, display = display, outputReg = outputReg)

    # caustic
    if maskCaustic:
        AXOFF_X = -15
        AXOFF_Y = 0
        xFovCen_ccd49 = 1036. + AXOFF_X # fov center in ccd49 pixel coords
        yFovCen_ccd49 = 4071. + AXOFF_Y
        fovCenterCoord = wcs49.pixelToSky(afwGeom.PointD(xFovCen_ccd49, yFovCen_ccd49))

        caustic = hscGhost.HscCausticGhostMaskTask()
        caustic.run(dataRef, exp, centerCoord = fovCenterCoord, display = display, outputReg = outputReg)

    if display:
        import lsst.afw.display.ds9 as ds9 
        mask = afwImage.ImageU(exp.getMaskedImage().getMask().getArray())
        ds9.mtv(mask, frame = frame + 1, title = fileName)

    exp.writeFits('masked_' + os.path.basename(fileName))


if __name__ == '__main__':
    corrName = sys.argv[1]
    corrName49 = sys.argv[2]
    rerunDir = sys.argv[3]
    frame = int(sys.argv[4])

    main(corrName, corrName49, rerunDir, display = True, outputReg = True, frame = frame, maskMustache = True, maskCaustic = True)

